Imports System.ComponentModel
Imports System.Drawing.Drawing2D

<DefaultEvent("Winkel_Geaendert"), DefaultProperty("Winkel"), Description("Trackbar in Bogenform")> _
Public Class PfeilTrackBar
    Private m_BackBMP As Bitmap
    Private m_NummernBMP As Bitmap

    Private m_Winkel As Integer = 0
    Private m_PunktPositionenFuerRadius1(8) As PointF

    Private m_ControlBereich As RectangleF
    Private m_Radius As Single
    Private m_DrehPunkt As PointF

    Private m_AcceptedKeys As List(Of Keys)
    Private m_Ziffern As String = ""

    Private m_PfeilPen As Pen
    Private m_PfeilZiel As PointF

    Private m_ZiffernRect As Rectangle
    Private m_ActualInvalPath As GraphicsPath

    Public Event Winkel_Geaendert(ByVal NeuerWinkel As Integer)

    <Description("Liefert den angezeigten Winkel oder legt ihn fest.")> _
    Property Winkel() As Integer
        Get
            Return m_Winkel
        End Get
        Set(ByVal value As Integer)
            If m_Winkel <> value Then
                SetWinkel(value)
            End If
        End Set
    End Property

    Private Sub Positionen_Neu_Festlegen()
        Dim Zeichenbreite As Integer = Me.Width - 2
        Dim Zeichenhoehe As Integer = Me.Height - 2

        If (Zeichenhoehe - 8) > Zeichenbreite / 2 Then
            'Breite ist bestimmend
            m_Radius = CSng(Zeichenbreite) / 2.0F

        Else
            'Höhe ist bestimmend für Größe des Controls
            m_Radius = CSng(Zeichenhoehe) - 8.0F
        End If

        Dim ControlSize As New SizeF(m_Radius * 2.0F, m_Radius + 8.0F)
        m_ControlBereich = New RectangleF(CSng(Me.Width) / 2.0F - ControlSize.Width / 2.0F, CSng(Me.Height) / 2.0F - ControlSize.Height / 2.0F, ControlSize.Width, ControlSize.Height)
        m_DrehPunkt = New PointF(m_ControlBereich.Left + m_ControlBereich.Width / 2.0F, m_ControlBereich.Bottom - 8.0F)

        Dim ZiffernTextCenterPunkt As New Point(m_ControlBereich.Left + m_ControlBereich.Width / 2.0F, m_ControlBereich.Top + m_ControlBereich.Height / 3.0F * 2.0F)
        m_ZiffernRect = New Rectangle(ZiffernTextCenterPunkt.X - 20, ZiffernTextCenterPunkt.Y - 15, 40, 30)

        m_PfeilZiel = GetWinkelPunkt(m_Winkel, m_Radius - 12.0F, m_DrehPunkt)

    End Sub


    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(pe)



        'Benutzerdefinierten Zeichnungscode hier einfügen

        If m_BackBMP Is Nothing Then
            m_BackBMP = New Bitmap(Me.Width, Me.Height)
            Dim g As Graphics = Graphics.FromImage(m_BackBMP)

            g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality

            Dim TranspTeal As Color = Color.FromArgb(120, Color.Teal)
            Dim TranspBlue As Color = Color.FromArgb(180, Color.DarkBlue)
            Dim BorderPen As New Pen(TranspTeal)

            Dim halbKreisBasis As New RectangleF(m_ControlBereich.Left, m_ControlBereich.Top, m_ControlBereich.Width, m_ControlBereich.Width)
            Dim halbKreisBasisSmall As New RectangleF(halbKreisBasis.Location, halbKreisBasis.Size)

            halbKreisBasisSmall.Inflate(-16, -16)

            Dim gp As New Drawing2D.GraphicsPath
            gp.StartFigure()
            gp.AddArc(halbKreisBasis, -180.0F, 180.0F)
            gp.AddLine(halbKreisBasis.Right, m_ControlBereich.Bottom, halbKreisBasisSmall.Right, m_ControlBereich.Bottom)

            gp.AddArc(halbKreisBasisSmall, 0.0F, -180.0F)
            gp.AddLine(halbKreisBasisSmall.Left, m_ControlBereich.Bottom, halbKreisBasis.Left, m_ControlBereich.Bottom)
            gp.CloseFigure()

            Dim BackBrush As New Drawing2D.LinearGradientBrush(halbKreisBasis, Color.White, Color.Teal, 90.0F)
            g.FillPath(BackBrush, gp)
            g.DrawPath(BorderPen, gp)

            For I As Integer = 0 To 8
                Dim Pt As PointF = GetMittelpunkt(I, m_Radius - 8.0F, m_DrehPunkt)
                Dim Rect As New RectangleF(Pt.X - 4.0F, Pt.Y - 4.0F, 8.0F, 8.0F)
                Dim GradBrush As New Drawing2D.LinearGradientBrush(Rect, TranspTeal, TranspBlue, 45.0F)
                g.FillEllipse(GradBrush, Rect)
                g.DrawEllipse(BorderPen, Rect)

                GradBrush.Dispose()

                Pt = GetMittelpunkt(I, m_Radius - 24.0F, m_DrehPunkt)
                DrawCenterText(g, (-90 + I * 22.5).ToString, Pt, Me.Font, Brushes.OrangeRed)
            Next

            BackBrush.Dispose()
            BorderPen.Dispose()

        End If

        If m_NummernBMP Is Nothing Then
            m_NummernBMP = New Bitmap(m_ZiffernRect.Width, m_ZiffernRect.Height)

            Dim g As Graphics = Graphics.FromImage(m_NummernBMP)
            g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
            DrawNummernBMP(g)
        End If



        pe.Graphics.SmoothingMode = Drawing2D.SmoothingMode.HighQuality

        pe.Graphics.DrawImage(m_BackBMP, 0, 0, Me.Width, Me.Height)
        pe.Graphics.DrawLine(m_PfeilPen, m_DrehPunkt, m_PfeilZiel)
        pe.Graphics.DrawImage(m_NummernBMP, m_ZiffernRect)


        If Me.ContainsFocus Then
            ControlPaint.DrawFocusRectangle(pe.Graphics, New Rectangle(0, 0, Me.Width, Me.Height))
        End If

    End Sub

    Private Sub DrawNummernBMP(ByVal g As Graphics)
        Dim Ft As New Font(Me.Font.FontFamily, 18.0F, FontStyle.Regular, GraphicsUnit.Point)

        Dim Sz As SizeF = g.MeasureString(m_Winkel.ToString, Ft)
        Dim PointToDraw As New Point((m_ZiffernRect.Width - Sz.Width) \ 2, (m_ZiffernRect.Height - Sz.Height) \ 2.0F)

        Dim BackColor As Color = Color.FromArgb(180, Color.White)
        g.Clear(BackColor)

        Dim TextBrush As New SolidBrush(Color.OrangeRed)
        g.DrawString(m_Winkel.ToString, Ft, TextBrush, PointToDraw)
        TextBrush.Dispose()
    End Sub
    Private Sub DrawCenterText(ByVal g As Graphics, ByVal Text As String, ByVal Pt As PointF, ByVal Ft As Font, ByVal Brsh As Brush)
        Dim Sz As SizeF = g.MeasureString(Text, Ft)
        Dim PointToDraw As New PointF(Pt.X - Sz.Width / 2.0F, Pt.Y - Sz.Height / 2.0F)
        g.DrawString(Text, Ft, Brsh, PointToDraw)
    End Sub

    Private Function GetMittelpunkt(ByVal PunktPosition As Integer, ByVal Radius As Single, ByVal Drehmittelpunkt As PointF) As PointF
        Return New PointF(Drehmittelpunkt.X + m_PunktPositionenFuerRadius1(PunktPosition).X * Radius, Drehmittelpunkt.Y - m_PunktPositionenFuerRadius1(PunktPosition).Y * Radius)
    End Function

    Private Sub PfeilTrackBar_GotFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.GotFocus
        m_Ziffern = ""
        Invalidate()
    End Sub

    Private Sub PfeilTrackBar_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        If e.KeyCode >= Keys.D0 AndAlso e.KeyCode <= Keys.D9 OrElse _
                e.KeyCode >= Keys.NumPad0 AndAlso e.KeyCode <= Keys.NumPad9 Then
            Dim Key As Keys = CType(e.KeyCode, Keys)
            Dim Str As String = Key.ToString
            m_Ziffern &= Str.Chars(Str.Length - 1)
        ElseIf e.KeyCode = Keys.Subtract OrElse e.KeyCode = Keys.OemMinus Then
            m_Ziffern = "-"
        ElseIf e.KeyCode = Keys.Enter Then
            If m_Ziffern <> "" Then
                Try
                    Dim Zahl As Integer = CInt(m_Ziffern)
                    SetWinkel(Zahl)
                Catch ex As Exception

                End Try
            End If
            m_Ziffern = ""
        Else
            If e.KeyCode = Keys.Right Then
                SetWinkel(m_Winkel + 1)
            ElseIf e.KeyCode = Keys.Left Then
                SetWinkel(m_Winkel - 1)
            ElseIf e.KeyCode = Keys.Up Then
                If m_Winkel Mod 10 <> 0 Then
                    If m_Winkel > 0 Then
                        m_Winkel -= m_Winkel Mod 10
                        SetWinkel(m_Winkel + 10)
                    Else
                        SetWinkel(m_Winkel - m_Winkel Mod 10)
                    End If
                Else
                    SetWinkel(m_Winkel + 10)
                End If
            ElseIf e.KeyCode = Keys.Down Then
                If m_Winkel Mod 10 <> 0 Then
                    If m_Winkel < 0 Then
                        m_Winkel -= m_Winkel Mod 10
                        SetWinkel(m_Winkel - 10)
                    Else
                        SetWinkel(m_Winkel - m_Winkel Mod 10)
                    End If
                Else
                    SetWinkel(m_Winkel - 10)
                End If
            End If
            m_Ziffern = ""
        End If

    End Sub


    Protected Overrides Function IsInputKey(ByVal keyData As System.Windows.Forms.Keys) As Boolean
        If MyBase.IsInputKey(keyData) OrElse m_AcceptedKeys.Contains(keyData) Then
            Return True
        End If
        Return False
    End Function
    Private Sub PfeilTrackBar_LostFocus(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LostFocus
        Invalidate()
    End Sub

    Private Sub PfeilTrackBar_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        Me.Focus()
        SetWinkelByMouse(New PointF(e.X, e.Y))
    End Sub

    Private Sub PfeilTrackBar_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        If e.Button = Windows.Forms.MouseButtons.Left Then
            SetWinkelByMouse(New PointF(e.X, e.Y))
        End If
    End Sub

    Private Sub SetWinkel(ByVal value As Integer)
        If value > 90 Then
            value = 90
        ElseIf value < -90 Then
            value = -90
        End If
        If m_Winkel <> value Then
            m_Winkel = value
            m_NummernBMP = Nothing

            m_PfeilZiel = GetWinkelPunkt(m_Winkel, m_Radius - 12.0F, m_DrehPunkt)

            Dim OldPathToInvalidate As GraphicsPath = m_ActualInvalPath
            m_ActualInvalPath = GetActualInvalPath()

            Dim CompletePathToInvalidate As New GraphicsPath
            CompletePathToInvalidate.FillMode = Drawing2D.FillMode.Winding
            CompletePathToInvalidate.AddPath(m_ActualInvalPath, False)
            CompletePathToInvalidate.AddPath(OldPathToInvalidate, False)

            Invalidate(New System.Drawing.Region(CompletePathToInvalidate))
            RaiseEvent Winkel_Geaendert(m_Winkel)
        End If
    End Sub
    Private Function GetActualInvalPath() As System.Drawing.Drawing2D.GraphicsPath
        Dim gp As New System.Drawing.Drawing2D.GraphicsPath
        gp.FillMode = Drawing2D.FillMode.Winding
        Dim DrehpunktRect As New Rectangle(m_DrehPunkt.X - 10, m_DrehPunkt.Y - 10, 20, 20)
        Dim PfeilZielRect As New Rectangle(m_PfeilZiel.X - 10, m_PfeilZiel.Y - 10, 20, 20)

        gp.AddRectangle(DrehpunktRect)
        gp.AddRectangle(PfeilZielRect)

        Dim Points(3) As Point
        If m_DrehPunkt.X > m_PfeilZiel.X Then
            Points(0) = New Point(DrehpunktRect.Left, DrehpunktRect.Bottom)
            Points(1) = New Point(PfeilZielRect.Left, PfeilZielRect.Bottom)
            Points(2) = New Point(PfeilZielRect.Right, PfeilZielRect.Top)
            Points(3) = New Point(DrehpunktRect.Right, DrehpunktRect.Top)
        Else
            Points(0) = New Point(DrehpunktRect.Left, DrehpunktRect.Top)
            Points(1) = New Point(PfeilZielRect.Left, PfeilZielRect.Top)
            Points(2) = New Point(PfeilZielRect.Right, PfeilZielRect.Bottom)
            Points(3) = New Point(DrehpunktRect.Right, DrehpunktRect.Bottom)
        End If

        gp.AddPolygon(Points)
        gp.AddRectangle(m_ZiffernRect)
        Return gp
    End Function

    Private Sub SetWinkelByMouse(ByVal MousePosition As PointF)
        If MousePosition.Y > m_DrehPunkt.Y Then

            If MousePosition.X < m_DrehPunkt.X Then
                SetWinkel(-90)
            Else
                SetWinkel(90)
            End If

        Else
            'Den realen Punkt für ein x,y-Koordinatensystem holen
            Dim Koordinatenpunkt As New PointF(MousePosition.X - m_DrehPunkt.X, m_DrehPunkt.Y - MousePosition.Y)

            '(tan alpa = Gegenkathete/Ankathete)
            Dim w1 As Single = Math.Atan(Koordinatenpunkt.X / Koordinatenpunkt.Y)
            'w1 ist der Winkel im Bogenmass, darum zurückrechnen

            Dim NewWinkel As Single = w1 * 180.0F / Math.PI
            If NewWinkel < -90.0F Then
                NewWinkel = -90.0F
            ElseIf NewWinkel > 90.0F Then
                NewWinkel = 90.0F
            End If

            SetWinkel(NewWinkel)
        End If

    End Sub

    Private Sub PfeilTrackBar_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        If Me.Width < 40 Then
            Me.Width = 40
        End If
        If Me.Height < 40 Then
            Me.Height = 40
        End If
        Positionen_Neu_Festlegen()
        m_BackBMP = Nothing
        m_NummernBMP = Nothing
  
        m_ActualInvalPath = New GraphicsPath
        m_ActualInvalPath.FillMode = FillMode.Winding
        m_ActualInvalPath.AddRectangle(New Rectangle(0, 0, Me.Width, Me.Height))
        Invalidate()
    End Sub

    Private Function GetWinkelPunkt(ByVal Winkel As Integer, ByVal Radius As Single, ByVal Drehmittelpunkt As PointF) As PointF

        Dim Bogenmass As Single = Math.PI * (90.0F - Winkel) / 180.0F
        Dim y As Single = Math.Sin(Bogenmass)
        Dim x As Single = Math.Cos(Bogenmass)
        Return New PointF(Drehmittelpunkt.X + x * Radius, Drehmittelpunkt.Y - y * Radius)
    End Function

    Public Sub New()

        ' Dieser Aufruf ist für den Windows Form-Designer erforderlich.
        InitializeComponent()

        ' Fügen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.
        Me.SetStyle(ControlStyles.DoubleBuffer _
           Or ControlStyles.UserPaint _
           Or ControlStyles.AllPaintingInWmPaint, _
           True)
        Me.MinimumSize = New Size(40, 40)
        For I As Integer = 0 To 8
            Dim Bogenmass As Single = Math.PI * (180.0F - CSng(I) * 22.5F) / 180.0F
            Dim y As Single = Math.Sin(Bogenmass)
            Dim x As Single = Math.Cos(Bogenmass)
            m_PunktPositionenFuerRadius1(I) = New PointF(x, y)
        Next
        m_AcceptedKeys = New List(Of Keys)
        m_AcceptedKeys.Add(Keys.Right)
        m_AcceptedKeys.Add(Keys.Left)
        m_AcceptedKeys.Add(Keys.Up)
        m_AcceptedKeys.Add(Keys.Down)

        Dim TranspBlue As Color = Color.FromArgb(180, Color.DarkBlue)

        m_PfeilPen = New Pen(TranspBlue, 3.5F)

        m_PfeilPen.CustomEndCap = New System.Drawing.Drawing2D.AdjustableArrowCap(3.0F, 4.0F)
        m_PfeilPen.StartCap = Drawing2D.LineCap.RoundAnchor


    End Sub




End Class
